// CustomPluginImpl.cpp  


#include "stdafx.h"
#include "resource.h"
#include "TrajectoryControl.h"
#include "Mach4View.h"
#include "Engine.h"
#include "rs274ngc.h"
#include "XMLProfile.h"

#include "CustomPluginImpl.h"
#include "PluginTestDlg.h"
#include "Utility.h"
#include "dbg.h"
#include "video.h"

#include <mmsystem.h>
#include <math.h> 

// ===========================================================================
// 
// Global variables for this code 
//
// ===========================================================================

// extern CMach4View *MachView;

CString DefDir ;
CString Profile ;

//============================================================================
//
// Here are the routines which are called by Mach3
// They are the core of every plugin
//
//============================================================================

// local variables

static BOOL pluginEnabled = FALSE ;  // TRUE after Mach resttarted with plugin enabled
static int isOpen = FALSE ;  // initialize as closed

//----------------------------------------------------------------------------

void myNotify ( int msg )
{
	DbgMsg(("CustomPluginImpl myNotify entry"));

    char tmp[81] ;
    sprintf ( tmp, "msg=0x%x", msg ) ;
    DbgMsg((tmp));

//	MessageBox ( NULL, tmp, "myNotify(int msg) entry", MB_OK ) ;

    if ( msg == 0xe )
	{
        DbgMsg(("Destroy Scan3d dialog box here"));
//		MessageBox ( NULL, "calling myCleanUp()from myNotify()", "", MB_OK ) ; 
		myCleanUp() ;
	}

//	MessageBox ( NULL, tmp, "myNotify(int msg) exit", MB_OK ) ;
	DbgMsg(("CustomPluginImpl myNotify exit"));
}

//----------------------------------------------------------------------------

// Used for destruction of variables prior to exit.
// Called as Mach3 shuts down. 
// but only if plugin is enabled

void myCleanUp()
{
	DbgMsg(("CustomPluginImpl myCleanUp entry"));

//	MessageBox ( NULL, "myCleanUp()", "entry", MB_OK ) ;

	DestroyVideoWindow() ;
	CloseScan3d() ;
    isOpen = FALSE ;  // mark as closed

//	MessageBox ( NULL, "myCleanUp()", "exit", MB_OK ) ;

	DbgMsg(("CustomPluginImpl myCleanUp exit"));
}

//----------------------------------------------------------------------------

// initial access to Mach profile when enumerating available plugins
// Returns the second half on the "pluging id" for the list in Operator
// CustomPlugin adds the file name at the start of the string

CString	myProfileInit(CString name, CXMLProfile *DevProf)	
{
	DbgMsg(("CustomPluginImpl myProfileInit entry"));

	//this gets the default directory DefDir in which Mach3 is located.  and the profile name ex. "Mach3Mill"

	DefDir = DevProf->GetProfileString("Preferences","DefDir","C:\\Mach3\\");
//MessageBox ( NULL, DefDir, "Default Directory", MB_OK ) ;
    Profile = DevProf->GetProfileString("Preferences","Profile","Mach3Mill");
//MessageBox ( NULL, Profile, "Profile", MB_OK ) ;

	DbgMsg(("CustomPluginImpl myProfileInit exit"));

	return(" PlugIn Ver 1.02 - Tom Hubin 11 May 2009");
}

//----------------------------------------------------------------------------

// called during Mach initialisation. You can influence subsequent init by actions here
// **** Not used in typical device plugin

void myInitControl()
{
	DbgMsg(("CustomPluginImpl myInitControl entry"));

//AfxMessageBox("CustomPlugin myInitControl entry");

	pluginEnabled = FALSE ;  // intialize as not enabled

//AfxMessageBox("CustomPlugin myInitControl exit");

	DbgMsg(("CustomPluginImpl myInitControl exit"));
}

//----------------------------------------------------------------------------

void myPostInitControl()
{
	// called when mach fully set up so all data can be used but initialization  outcomes not affected

	DbgMsg(("CustomPluginImpl myPostInitControl entry"));

	pluginEnabled = TRUE ;  // only here if plugin enabled before Mach starts 
 
	DbgMsg(("CustomPluginImpl myPostInitControl exit"));
}

//----------------------------------------------------------------------------

// Called to configure the device
// Has read/write access to Mach XML profile to remember what it needs to.

void myConfig(CXMLProfile *DevProf)
{
	int rtc = 0 ;

	static BOOL fIsWorking = FALSE ;	// entry counter
	if ( fIsWorking == TRUE ) return ;  // prevent recursion
	fIsWorking = TRUE ;  // flag as work in progress

// place your source code below

	DbgMsg(("CustomPluginImpl myConfig() entry"));

//	if ( !pluginEnabled )
//	{
//        MessageBox ( NULL, "Enable plugin, then restart Mach", NULL, MB_OK ) ;
//		rtc = 2 ;
//		goto quit ;
//	}

    if ( OpenScan3d() == 0 )
		isOpen = TRUE ;
	else
		isOpen = FALSE ;

// place your source code above

quit:
	fIsWorking = FALSE ;  // flag as finished
	DbgMsg(("CustomPluginImpl myConfig() exit"));
	return ;
}

//----------------------------------------------------------------------------

// 10Hz update loop

void myUpdate()
{
	if ( isOpen == FALSE )  //  must be properly initialized first 
		return ;

//DbgMsg(("myUpdate() entry"));
    static BOOL fIsWorking = FALSE ;	// entry counter
	if ( fIsWorking == TRUE ) return ;  // prevent recursion
	fIsWorking = TRUE ;  // flag as work in progress

// place your source code below

//    UpdateScan3d() ; // grab a frame and process it

// place your source code above

	fIsWorking = FALSE ;  // flag as finished
//DbgMsg(("myUpdate() exit"));
	return ;
}

//----------------------------------------------------------------------------

// called at 40Hz. This is used for Higher Res timing needs. 
// only use as required, it can use alot of system resources. 

void myHighSpeedUpdate()
{
//DbgMsg(("myHighSpeedUpdate() entry"));
    static BOOL fIsWorking = FALSE ;	// entry counter
	if ( fIsWorking ) return ;  // prevent recursion
	fIsWorking = TRUE ;  // flag as work in progress

// place your source code below

// place your source code above

	fIsWorking = FALSE ; //  flag as finished 
	return ;
}

//----------------------------------------------------------------------------

// ======================================================================================================
//
// The Helpers follow
//
//=======================================================================================================

// put your helper routines here

 