// PluginTestDlg.cpp : implementation file

//    Program Name: Scan3d    Plugin for 3d video scanning using Mach3.
//    Copyright (C) 2008,2009  Thomas Hubin

//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.

//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License gpl.txt for more details.

//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

//    Author: Tom Hubin, thubin@earthlink.net
//    13151 Larchdale Rd, #5
//    Laurel, MD 20708-1719

#include "stdafx.h"
#include "resource.h"
#include "CustomPlugin.h"
#include "PluginTestDlg.h"
#include "dbg.h"
#include ".\plugintestdlg.h"
#include <float.h>

#include "video.h"

// CPluginTestDlg dialog

IMPLEMENT_DYNAMIC(CPluginTestDlg, CDialog)
CPluginTestDlg::CPluginTestDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CPluginTestDlg::IDD, pParent)

	, xFirst(-0.5)
	, xLast(0.5)
	, xStep(0.01)

	, yFirst(-0.5)
	, yLast(0.5)
	, yStep(0.5)

	, zFirst(-0.5)
	, zLast(0.5)
	, zStep(0.25)

	, feed(1)

	, xOrder(1)
	, yOrder(2)
	, zOrder(3)

	, fIsPauseVideo(false)
	, fEnableScan(false)
	, videoThreshold(0)
	, videoMirror(FALSE)
	, videoRotation(0)
	, sampleInterval(1)

	, hfov(0)
	, vfov(0)
	, elevation(0)
	, azimuth(0)
	, xScale(1)
	, yScale(1)
	, zScale(1)
{
}

CPluginTestDlg::~CPluginTestDlg()
{
DbgMsg(("CPluginTestDlg destructor entry"));
//    DestroyVideoWindow() ;
DbgMsg(("CPluginTestDlg destructor exit"));
}

void CPluginTestDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);

	DDX_Text(pDX, IDC_XORDER, xOrder);
	DDV_MinMaxUInt(pDX, xOrder, 0, 3);
	DDX_Text(pDX, IDC_XFIRST, xFirst);
	DDX_Text(pDX, IDC_XLAST, xLast);
	DDX_Text(pDX, IDC_XSTEP, xStep);
	DDV_MinMaxDouble(pDX, xStep, 0, DBL_MAX);

	DDX_Text(pDX, IDC_YORDER, yOrder);
	DDV_MinMaxUInt(pDX, yOrder, 0, 3);
	DDX_Text(pDX, IDC_YFIRST, yFirst);
	DDX_Text(pDX, IDC_YLAST, yLast);
	DDX_Text(pDX, IDC_YSTEP, yStep);
	DDV_MinMaxDouble(pDX, yStep, 0, DBL_MAX);

	DDX_Text(pDX, IDC_ZORDER, zOrder);
	DDV_MinMaxUInt(pDX, zOrder, 0, 3);
	DDX_Text(pDX, IDC_ZFIRST, zFirst);
	DDX_Text(pDX, IDC_ZLAST, zLast);
	DDX_Text(pDX, IDC_ZSTEP, zStep);
	DDV_MinMaxDouble(pDX, zStep, 0, DBL_MAX);

	DDX_Text(pDX, IDC_FEED, feed);
	DDV_MinMaxDouble(pDX, feed, 0, DBL_MAX);

	DDX_Text(pDX, IDC_THRESHOLD, videoThreshold);
	DDV_MinMaxUInt(pDX, videoThreshold, 0, 765);
	DDX_Check(pDX, IDC_MIRROR, videoMirror);
	DDX_Radio(pDX, IDC_VIDEOROTATE000, videoRotation);
	DDX_Text(pDX, IDC_SAMPLEINTERVAL, sampleInterval);
	DDV_MinMaxUInt(pDX, sampleInterval, 1, 1000);
	//	DDX_Control(pDX, IDC_THRESHOLD3, threshold);
	DDX_Text(pDX, IDC_HFOV, hfov);
	DDX_Text(pDX, IDC_VFOV, vfov);
	DDX_Text(pDX, IDC_ELEVATION, elevation);
	DDX_Text(pDX, IDC_AZIMUTH, azimuth);
	DDX_Text(pDX, IDC_XSCALE, xScale);
	DDX_Text(pDX, IDC_YSCALE, yScale);
	DDX_Text(pDX, IDC_ZSCALE, zScale);
}


BEGIN_MESSAGE_MAP(CPluginTestDlg, CDialog)
	ON_BN_CLICKED(IDOK, OnBnClickedOk)
	ON_BN_CLICKED(IDCANCEL, OnBnClickedCancel)
	ON_BN_CLICKED(IDC_VIDEOSOURCE, OnBnClickedVideosource)
	ON_BN_CLICKED(IDC_VIDEOFORMAT, OnBnClickedVideoformat)
	ON_BN_CLICKED(IDC_VIDEOPAUSE, OnBnClickedVideopause)
	ON_BN_CLICKED(IDC_VIDEORESUME, OnBnClickedVideoresume)
	ON_BN_CLICKED(IDC_XSWAP, OnBnClickedXswap)
	ON_BN_CLICKED(IDC_YSWAP, OnBnClickedYswap)
	ON_BN_CLICKED(IDC_ZSWAP, OnBnClickedZswap)
	ON_BN_CLICKED(IDC_ToClipboard, OnBnClickedToclipboard)
	ON_BN_CLICKED(IDC_DrawTest1, OnBnClickedDrawtest1)
	ON_BN_CLICKED(IDC_SCANSTART, OnBnClickedScanstart)
	ON_BN_CLICKED(IDC_SCANSTOP, OnBnClickedScanstop)
	ON_BN_CLICKED(IDC_XFIRSTDRO, OnBnClickedXfirstdro)
	ON_BN_CLICKED(IDC_XLASTDRO, OnBnClickedXlastdro)
	ON_BN_CLICKED(IDC_YFIRSTDRO, OnBnClickedYfirstdro)
	ON_BN_CLICKED(IDC_YLASTDRO, OnBnClickedYlastdro)
	ON_BN_CLICKED(IDC_ZFIRSTDRO, OnBnClickedZfirstdro)
	ON_BN_CLICKED(IDC_ZLASTDRO, OnBnClickedZlastdro)
	ON_BN_CLICKED(IDC_VIDEOROTATE000, OnBnClickedVideorotate000)
	ON_BN_CLICKED(IDC_VIDEOROTATE090, OnBnClickedVideorotate090)
	ON_BN_CLICKED(IDC_VIDEOROTATE180, OnBnClickedVideorotate180)
	ON_BN_CLICKED(IDC_VIDEOROTATE270, OnBnClickedVideorotate270)
	ON_BN_CLICKED(IDC_MIRROR, OnBnClickedMirror)
	ON_BN_CLICKED(IDC_SAVE, OnBnClickedSave)
	ON_EN_KILLFOCUS(IDC_THRESHOLD, OnEnKillfocusThreshold)
	ON_EN_KILLFOCUS(IDC_SAMPLEINTERVAL, OnEnKillfocusSampleinterval)
	ON_EN_KILLFOCUS(IDC_XORDER, OnEnKillfocusXorder)
	ON_EN_KILLFOCUS(IDC_YORDER, OnEnKillfocusYorder)
	ON_EN_KILLFOCUS(IDC_ZORDER, OnEnKillfocusZorder)
	ON_EN_KILLFOCUS(IDC_XFIRST, OnEnKillfocusXfirst)
	ON_EN_KILLFOCUS(IDC_YFIRST, OnEnKillfocusYfirst)
	ON_EN_KILLFOCUS(IDC_ZFIRST, OnEnKillfocusZfirst)
	ON_EN_KILLFOCUS(IDC_XLAST, OnEnKillfocusXlast)
	ON_EN_KILLFOCUS(IDC_YLAST, OnEnKillfocusYlast)
	ON_EN_KILLFOCUS(IDC_ZLAST, OnEnKillfocusZlast)
	ON_EN_KILLFOCUS(IDC_XSTEP, OnEnKillfocusXstep)
	ON_EN_KILLFOCUS(IDC_YSTEP, OnEnKillfocusYstep)
	ON_EN_KILLFOCUS(IDC_ZSTEP, OnEnKillfocusZstep)
	ON_BN_CLICKED(IDC_CALCSUMS, OnBnClickedCalcsums)
	ON_BN_CLICKED(IDC_CALCTESTCLOUD, OnBnClickedCalctestcloud)
	ON_BN_CLICKED(IDC_CALSTART, OnBnClickedCalstart)
	ON_EN_KILLFOCUS(IDC_FEED, OnEnKillfocusFeed)
	ON_BN_CLICKED(IDC_VIDEODISPLAY, OnBnClickedVideodisplay)
	ON_BN_CLICKED(IDC_CALCFOV, OnBnClickedCalcfov)
	ON_EN_CHANGE(IDC_VFOV, OnEnChangeVfov)
	ON_EN_KILLFOCUS(IDC_XSCALE, OnEnKillfocusXscale)
	ON_EN_KILLFOCUS(IDC_YSCALE, OnEnKillfocusYscale)
	ON_EN_KILLFOCUS(IDC_ZSCALE, OnEnKillfocusZscale)
	ON_BN_CLICKED(IDC_VIDEODEVICE, OnBnClickedVideodevice)
END_MESSAGE_MAP()


// CPluginTestDlg message handlers


void CPluginTestDlg::OnBnClickedOk()
{
	// TODO: Add your control notification handler code here

	DbgMsg(("CPluginTestDlg OnBnClickedOk entry"));

	if ( UpdateData(TRUE) )
	{
		char* iniFileName = "scan3d.ini" ;
		SaveScan3dData ( iniFileName ) ;
		DestroyVideoWindow() ;	// kill the video window
        CloseScan3d() ;  // kill this dialog box
	}

	DbgMsg(("CPluginTestDlg OnBnClickedOk exit"));

}

void CPluginTestDlg::OnBnClickedCancel()
{
	// TODO: Add your control notification handler code here

	DbgMsg(("CPluginTestDlg OnBnClickedCancel entry"));

	DestroyVideoWindow() ;  // kill the video window
    CloseScan3d() ;  // kill this dialog box
	DbgMsg(("CPluginTestDlg OnBnClickedCancel exit"));

}

void CPluginTestDlg::OnBnClickedSave()
{
	// TODO: Add your control notification handler code here

	if ( UpdateData(TRUE) )
	{
		char* iniFileName = "scan3d.ini" ;
		SaveScan3dData ( iniFileName ) ;
	}
}

void CPluginTestDlg::OnBnClickedVideosource()
{
	// TODO: Add your control notification handler code here

	DbgMsg(("CPluginTestDlg::OnBnClickedVideosource() entry"));

	ActivateVideoSourceDlg() ;  //  activate camera driver source dialog 

	DbgMsg(("CPluginTestDlg::OnBnClickedVideosource() exit"));
}

void CPluginTestDlg::OnBnClickedVideoformat()
{
	// TODO: Add your control notification handler code here

	DbgMsg(("CPluginTestDlg::OnBnClickedVideoformat() entry"));

	ActivateVideoFormatDlg() ;  //  activate camera driver format dialog

	DbgMsg(("CPluginTestDlg::OnBnClickedVideoformat() exit"));
}

void CPluginTestDlg::OnBnClickedVideodisplay()
{
	// TODO: Add your control notification handler code here

	DbgMsg(("CPluginTestDlg::OnBnClickedVideodisplay() entry"));

	ActivateVideoDisplayDlg() ;  //  activate camera driver display dialog

	DbgMsg(("CPluginTestDlg::OnBnClickedVideodisplay() exit"));
}

void CPluginTestDlg::OnBnClickedVideodevice()
{
	// TODO: Add your control notification handler code here

	DbgMsg(("CPluginTestDlg::OnBnClickedVideodevice() entry"));

	SelectCamera() ;  //  select camera from list

	DbgMsg(("CPluginTestDlg::OnBnClickedVideodevice() exit"));
}

void CPluginTestDlg::OnBnClickedVideopause()
{
	// TODO: Add your control notification handler code here

    DbgMsg(("CPluginTestDlg::OnBnClickedVideopause() entry"));

    fIsPauseVideo = TRUE ;  //  this will pause video updating

    DbgMsg(("CPluginTestDlg::OnBnClickedVideopause() exit"));

}

void CPluginTestDlg::OnBnClickedVideoresume()
{
	// TODO: Add your control notification handler code here

    DbgMsg(("CPluginTestDlg::OnBnClickedVideoresume() entry"));

	fIsPauseVideo = FALSE ;  //  this will resume video updating

    DbgMsg(("CPluginTestDlg::OnBnClickedVideoresume() exit"));

}

void CPluginTestDlg::OnBnClickedXswap()
{
	// TODO: Add your control notification handler code here

	if ( UpdateData(TRUE) )  //  make sure all data is valid
	{
        double tmp = xFirst ;  // swap the endpoints
		xFirst = xLast ;
		xLast = tmp ;
		UpdateData(FALSE) ;  // show the new data
	}
}

void CPluginTestDlg::OnBnClickedYswap()
{
	// TODO: Add your control notification handler code here

	if ( UpdateData(TRUE) )  //  make sure all data is valid
	{
        double tmp = yFirst ;  // swap the endpoints
		yFirst = yLast ;
		yLast = tmp ;
		UpdateData(FALSE) ;  // show the new data
	}
}

void CPluginTestDlg::OnBnClickedZswap()
{
	// TODO: Add your control notification handler code here

	if ( UpdateData(TRUE) )  //  make sure all data is valid
	{
        double tmp = zFirst ;  // swap the endpoints
		zFirst = zLast ;
		zLast = tmp ;
		UpdateData(FALSE) ;  // show the new data
	}
}


void CPluginTestDlg::OnBnClickedToclipboard()
{
	// TODO: Add your control notification handler code here

	SnapshotToClipboard() ;

}

void CPluginTestDlg::OnBnClickedDrawtest1()
{
	// TODO: Add your control notification handler code here

	DrawTest1() ;
    
}

void CPluginTestDlg::OnBnClickedScanstart()
{
	// TODO: Add your control notification handler code here

    if ( UpdateData(TRUE) )
	{
		fEnableScan = TRUE ;
	    DoScan() ;
	}
}
void CPluginTestDlg::OnBnClickedScanstop()
{
	// TODO: Add your control notification handler code here

	fEnableScan = FALSE ;

}

void CPluginTestDlg::OnBnClickedXfirstdro()
{
	// TODO: Add your control notification handler code her

	if ( UpdateData(TRUE) )  //  make sure all data is valid
	{
       	xFirst = GetDRO(800) ;
		UpdateData(FALSE) ;  // show the new data
	}
}

void CPluginTestDlg::OnBnClickedXlastdro()
{
	// TODO: Add your control notification handler code here

	if ( UpdateData(TRUE) )  //  make sure all data is valid
	{
       	xLast = GetDRO(800) ;
		UpdateData(FALSE) ;  // show the new data
	}
}

void CPluginTestDlg::OnBnClickedYfirstdro()
{
	// TODO: Add your control notification handler code here

	if ( UpdateData(TRUE) )  //  make sure all data is valid
	{
       	yFirst = GetDRO(801) ;
		UpdateData(FALSE) ;  // show the new data
	}
}

void CPluginTestDlg::OnBnClickedYlastdro()
{
	// TODO: Add your control notification handler code here

	if ( UpdateData(TRUE) )  //  make sure all data is valid
	{
       	yLast = GetDRO(801) ;
		UpdateData(FALSE) ;  // show the new data
	}
}

void CPluginTestDlg::OnBnClickedZfirstdro()
{
	// TODO: Add your control notification handler code here

	if ( UpdateData(TRUE) )  //  make sure all data is valid
	{
       	zFirst = GetDRO(802) ;
		UpdateData(FALSE) ;  // show the new data
	}
}

void CPluginTestDlg::OnBnClickedZlastdro()
{
	// TODO: Add your control notification handler code here

	if ( UpdateData(TRUE) )  //  make sure all data is valid
	{
       	zLast = GetDRO(802) ;
		UpdateData(FALSE) ;  // show the new data
	}
}

void CPluginTestDlg::OnBnClickedVideorotate000()
{
	// TODO: Add your control notification handler code here

	if ( UpdateData(TRUE) )
	{
        UpdateData(FALSE) ;
	}
}

void CPluginTestDlg::OnBnClickedVideorotate090()
{
	// TODO: Add your control notification handler code here

	if ( UpdateData(TRUE) )
	{
        UpdateData(FALSE) ;
	}
}

void CPluginTestDlg::OnBnClickedVideorotate180()
{
	// TODO: Add your control notification handler code here

	if ( UpdateData(TRUE) )
	{
        UpdateData(FALSE) ;
	}
}

void CPluginTestDlg::OnBnClickedVideorotate270()
{
	// TODO: Add your control notification handler code here

	if ( UpdateData(TRUE) )
	{
        UpdateData(FALSE) ;
	}
}

void CPluginTestDlg::OnBnClickedMirror()
{
	// TODO: Add your control notification handler code here

	if ( UpdateData(TRUE) )
	{
        UpdateData(FALSE) ;
	}
}

void CPluginTestDlg::OnEnKillfocusThreshold()
{
	// TODO: Add your control notification handler code here

	UpdateData(TRUE) ;
}

void CPluginTestDlg::OnEnKillfocusSampleinterval()
{
	// TODO: Add your control notification handler code here

	UpdateData(TRUE) ;
}

void CPluginTestDlg::OnEnKillfocusXorder()
{
	// TODO: Add your control notification handler code here

	UpdateData(TRUE) ;
}

void CPluginTestDlg::OnEnKillfocusYorder()
{
	// TODO: Add your control notification handler code here

	UpdateData(TRUE) ;
}

void CPluginTestDlg::OnEnKillfocusZorder()
{
	// TODO: Add your control notification handler code here

	UpdateData(TRUE) ;
}

void CPluginTestDlg::OnEnKillfocusXfirst()
{
	// TODO: Add your control notification handler code here

	UpdateData(TRUE) ;
}

void CPluginTestDlg::OnEnKillfocusYfirst()
{
	// TODO: Add your control notification handler code here

	UpdateData(TRUE) ;
}

void CPluginTestDlg::OnEnKillfocusZfirst()
{
	// TODO: Add your control notification handler code here

	UpdateData(TRUE) ;
}

void CPluginTestDlg::OnEnKillfocusXlast()
{
	// TODO: Add your control notification handler code here

	UpdateData(TRUE) ;
}

void CPluginTestDlg::OnEnKillfocusYlast()
{
	// TODO: Add your control notification handler code here

	UpdateData(TRUE) ;
}

void CPluginTestDlg::OnEnKillfocusZlast()
{
	// TODO: Add your control notification handler code here

	UpdateData(TRUE) ;
}

void CPluginTestDlg::OnEnKillfocusXstep()
{
	// TODO: Add your control notification handler code here

	UpdateData(TRUE) ;
}

void CPluginTestDlg::OnEnKillfocusYstep()
{
	// TODO: Add your control notification handler code here

	UpdateData(TRUE) ;
}

void CPluginTestDlg::OnEnKillfocusZstep()
{
	// TODO: Add your control notification handler code here

	UpdateData(TRUE) ;
}

void CPluginTestDlg::OnBnClickedCalcsums()
{
	// TODO: Add your control notification handler code here

	if ( UpdateData(TRUE) )
	{
		FILE *inFile = fopen ( "Scan3dCal02.pts", "r" ) ;
		if ( inFile == NULL )
		{
            MessageBox ( "Unable to open input file of cal points", NULL, MB_OK ) ; 
		}
		else
		{
            CalcCoef ( inFile ) ;
            fclose ( inFile ) ;
		    inFile = NULL ;
		}
	}
}

void CPluginTestDlg::OnBnClickedCalctestcloud()
{
	// TODO: Add your control notification handler code here

	if ( UpdateData(TRUE) )
	{
        MakeTestCloud ( "Scan3dCal02.pts", "Scan3dCal02.xyz") ;
	}
}

void CPluginTestDlg::OnBnClickedCalstart()
{
	// TODO: Add your control notification handler code here

	if ( UpdateData(TRUE) )
	{
		fEnableScan = TRUE ;
        DoCal() ;
	}
}

void CPluginTestDlg::OnEnKillfocusFeed()
{
	// TODO: Add your control notification handler code here

    UpdateData(TRUE) ;
}


void CPluginTestDlg::OnBnClickedCalcfov()
{
	// TODO: Add your control notification handler code here

	if ( UpdateData(TRUE) )  //  make sure all data is valid
	{
       	CalcFov() ;
		UpdateData(FALSE) ;  // show the new data
	}
}

void CPluginTestDlg::OnEnChangeVfov()
{
	// TODO:  If this is a RICHEDIT control, the control will not
	// send this notification unless you override the CDialog::OnInitDialog()
	// function and call CRichEditCtrl().SetEventMask()
	// with the ENM_CHANGE flag ORed into the mask.

	// TODO:  Add your control notification handler code here
}

void CPluginTestDlg::OnEnKillfocusXscale()
{
	// TODO: Add your control notification handler code here

	UpdateData(TRUE) ;
}

void CPluginTestDlg::OnEnKillfocusYscale()
{
	// TODO: Add your control notification handler code here

	UpdateData(TRUE) ;
}

void CPluginTestDlg::OnEnKillfocusZscale()
{
	// TODO: Add your control notification handler code here

	UpdateData(TRUE) ;
}

